@@ -3,7 +3,7 @@ package Crypt::JWT;
 use strict;
 use warnings;
 
-our $VERSION = '0.030';
+our $VERSION = '0.033';
 
 use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw(decode_jwt encode_jwt)] );
@@ -168,6 +168,17 @@ sub _verify_claims {
 
   return if $args{ignore_claims};
 
+  if (ref($payload) ne 'HASH') {
+    # https://github.com/DCIT/perl-Crypt-JWT/issues/31
+    # payload needs to be decoded into a HASH for checking any verify_XXXX
+    for my $claim (qw(exp nbf iat iss sub aud jti)) {
+      if (defined $args{"verify_$claim"} && $args{"verify_$claim"} != 0) {
+        croak "JWT: cannot check verify_$claim (payload not decoded JSON/HASH)";
+      }
+    }
+    return; # nothing to check
+  }
+
   my $leeway = $args{leeway} || 0;
   my $now = time;
 
@@ -527,7 +538,7 @@ sub _decode_jwe {
   my $payload = _decrypt_jwe_payload($cek, $header->{enc}, $aad, $ct, $iv, $tag);
   $payload = _payload_unzip($payload, $header->{zip}) if $header->{zip};
   $payload = _payload_dec($payload, $args{decode_payload});
-  _verify_claims($payload, %args) if ref $payload eq 'HASH'; # croaks on error
+  _verify_claims($payload, %args); # croaks on error
   return ($header, $payload);
 }
 
@@ -683,7 +694,7 @@ sub _decode_jws {
   croak "JWS: invalid payload part" if $b64u_payload && !$payload;
   $payload = _payload_unzip($payload, $header->{zip}) if $header->{zip};
   $payload = _payload_dec($payload, $args{decode_payload});
-  _verify_claims($payload, %args) if ref $payload eq 'HASH'; # croaks on error
+  _verify_claims($payload, %args); # croaks on error
   $header = { %$unprotected_header, %$header }; # merge headers
   return ($header, $payload);
 }
@@ -693,7 +704,7 @@ sub encode_jwt {
 
   croak "JWT: missing 'alg'" unless $args{alg};
   my $ser = $args{serialization} || 'compact';
-  if ($args{alg} =~ /^(none|EdDSA|((HS|RS|PS|ES)(512|384|256)))$/) {
+  if ($args{alg} =~ /^(none|EdDSA|(HS|RS|PS)(256|384|512)|ES(256|256K|384|512))$/) {
     ###JWS
     my ($b64u_header, $b64u_payload, $b64u_signature) = _encode_jws(%args);
     if ($ser eq 'compact') { # https://tools.ietf.org/html/rfc7515#section-7.1
@@ -709,7 +720,7 @@ sub encode_jwt {
       croak "JWT: unsupported JWS serialization '$ser'";
     }
   }
-  else {
+  elsif ($args{alg} =~ /^(dir|A(128|192|256)KW|A(128|192|256)GCMKW|PBES2-(HS256\+A128KW|HS384\+A192KW|HS512\+A256KW)|RSA-OAEP|RSA-OAEP-256|RSA1_5|ECDH-ES\+A(128|192|256)KW|ECDH-ES)$/) {
     ### JWE
     my ($b64u_header, $b64u_ecek, $b64u_iv, $b64u_ct, $b64u_tag, $b64u_aad) = _encode_jwe(%args);
     if ($ser eq 'compact') { # https://tools.ietf.org/html/rfc7516#section-7.1
@@ -738,6 +749,9 @@ sub encode_jwt {
       croak "JWT: unsupported JWE serialization '$ser'";
     }
   }
+  else {
+      croak "JWT: unexpected alg '$args{alg}'";
+  }
 }
 
 sub decode_jwt {
